from .gpt_detector import GPTDetector
from .object_detection import ObjectDetection
from .image_captioning import ImageCaptioning
#from .utils import *

__title__ = "Wyzard"
__version__ = "1.0"
__authors__ = "LyQuid"
__license__ = "MIT License"
__copyright__ = "Copyright 2023-present LyQuid"
